from django.db import models

# Example:
#
# {'AvailabilityZone': '',
#  'Cost': '0.00000120',
#  'InvoiceID': 'Estimated',
#  'ItemDescription': '$0.004 per 10,000 GET and all other requests',
#  'LinkedAccountId': '572107770384',
#  'Operation': 'ReadACL',
#  'PayerAccountId': '572107770384',
#  'PricingPlanId': '349114',
#  'ProductName': 'Amazon Simple Storage Service',
#  'Rate': '0.0000004000',
#  'RateId': '2016509',
#  'RecordId': '16550049848010735519421357', 
#  'RecordType': 'LineItem',
#  'ReservedInstance': 'N',
#  'ResourceId': 'aws-billing-info-bucket',
#  'SubscriptionId': '46364503',
#  'UsageEndDate': '2014-02-03 13:00:00',
#  'UsageQuantity': '3.00000000',
#  'UsageStartDate': '2014-02-03 12:00:00',
#  'UsageType': 'USW2-Requests-Tier2'}

class BillingRecord(models.Model):

    availablility_zone = models.CharField('Availability Zone', null=True, blank=True, max_length=255)
    cost = models.FloatField('Cost', null=True, blank=True)
    invoice_id = models.CharField('Invoice ID', null=True, blank=True, max_length=255)
    item_description = models.CharField('Item Description', null=True, blank=True, max_length=255)
    linked_account_id = models.BigIntegerField('Linked Account ID', null=True, blank=True)
    operation = models.CharField('Read ACL', null=True, blank=True, max_length=255)
    payer_account_id = models.BigIntegerField('Payer Account ID', null=True, blank=True)
    pricing_plan_id = models.IntegerField('Pricing Plan ID', null=True, blank=True)
    product_name = models.CharField('Product Name', null=True, blank=True, max_length=255)
    rate = models.FloatField('Rate', null=True, blank=True)
    rate_id = models.IntegerField('Rate ID', null=True, blank=True)
    record_id = models.CharField('Record ID', null=False, blank=False, unique=True, max_length=255) # This is the magic
    record_type = models.CharField('Record Type', null=True, blank=True, max_length=255)
    reserved_instance = models.CharField('Reserved Instance', null=True, blank=True, max_length=255)
    resource_id = models.CharField('Resource ID', null=True, blank=True, max_length=255)
    subscription_id = models.IntegerField('Subscription ID', null=True, blank=True)
    usage_end_date = models.DateTimeField('Usage End Date', null=True, blank=True)
    usage_quantity = models.FloatField('Usage Quantity', null=True, blank=True)
    usage_start_date = models.DateTimeField('Usage Start Date', null=True, blank=True)
    usage_type = models.CharField('Usage Type', null=True, blank=True, max_length=255)

    def __unicode__(self):
        return "BillingRecord: " + str(self.record_id) + " " + str(self.resource_id)

    def save(self):

        super(BillingRecord, self).save()
        return True



from django.core.management.base import BaseCommand
from django.conf import settings as aws_settings

try:
    from libs.django_aws_billing.aws_billing import aws_billing
except Exception, e:
    import aws_billing.aws_billing as aws_billing

import time
import sys
import csv
import os
import zipfile

from datetime import datetime
from pprint import pprint
from collections import defaultdict
from boto import connect_s3

class Command(BaseCommand):
    args = '<filename>'
    help = 'Print AWS Costs.'

    def handle(self, *args, **options):

        all_costs = aws_billing.get_all_costs_by_resource()
        for k, v in all_costs.iteritems():
            print str(k) + ": \t\t\t$" + str(v)

        print "------------------------"
        print "Total cost: \t\t\t$" + str(aws_billing.get_total_cost())



from django.core.management.base import BaseCommand
from django.conf import settings as aws_settings

try:
    from libs.django_aws_billing.aws_billing.models import BillingRecord
except Exception, e:
    from aws_billing.models import BillingRecord

import csv
import os
import zipfile
try:
      from cStringIO import StringIO
except:
      from StringIO import StringIO

from datetime import datetime
from pprint import pprint
from boto import connect_s3

class Command(BaseCommand):
    args = '<filename>'
    help = 'Parse an AWS Billing CSV into aws_billing format.'

    def handle(self, *args, **options):

        filename = None
        for arg in args:
            filename = arg
            fd = open(filename)
        if not filename:
            fd = retrieve_remote_stats(aws_settings.AWS_ACCOUNT_ID.replace('-', ''), aws_settings.AWS_BILLING_BUCKET)

        reader = csv.reader(fd, delimiter=',', quotechar='"')
        legend = None

        for row in reader:

            # First, get the legend from the very first row.
            if not legend:
                legend = row
                continue

            # Parse the row into a Python dict using the legend.
            data = dict(zip(legend, row))
            record_id = data['RecordId']

            # This is a summary record, not a billing record.
            if not record_id:
                continue

            if BillingRecord.objects.filter(record_id=record_id).exists():
                continue
            else:
                # Here we go!
                try:
                    record = BillingRecord()
                    record.availablility_zone = data['AvailabilityZone']
                    try:
                        record.cost = data['Cost']
                    except Exception, e:
                        record.cost = data['BlendedCost']
                    record.invoice_id = data['InvoiceID']
                    record.item_description = data['ItemDescription']
                    record.linked_account_id = data['LinkedAccountId']
                    record.operation = data['Operation']
                    record.payer_account_id = data['PayerAccountId']
                    record.pricing_plan_id = data['PricingPlanId']
                    record.product_name = data['ProductName']
                    try:
                        record.rate = data['Rate']
                    except Exception, e:
                        record.rate = data['BlendedRate']
                    record.rate_id = data['RateId']
                    record.record_id = data['RecordId'] # This is the magic
                    record.record_type = data['RecordType']
                    record.reserved_instance = data['ReservedInstance']
                    record.resource_id = data['ResourceId']
                    record.subscription_id = data['SubscriptionId']
                    record.usage_end_date = data['UsageEndDate']
                    record.usage_quantity = data['UsageQuantity']
                    record.usage_start_date= data['UsageStartDate']
                    record.usage_type = data['UsageType']
                    record.save()
                except Exception, e:
                    print "There was an error saving your record: "
                    print e
                    print "Here is the problem data: "
                    pprint(data)
                    print "And here's a shell: "
                    import pdb
                    pdb.set_trace() 

def retrieve_remote_stats(account, bucket_name, month=None, tmp_dir='.', as_file=False):
    month = month or datetime.now().strftime('%Y-%m')
    fn = "%s-aws-billing-detailed-line-items-with-resources-and-tags-%s.csv" % (account, month)
    remote_fn = "s3://%s/%s.zip" % (bucket_name, fn)
    s3 = connect_s3(aws_settings.AWS_ACCESS_KEY, aws_settings.AWS_SECRET_ACCESS_KEY)
    bucket = s3.get_bucket(bucket_name)
    key = bucket.get_key(fn+'.zip')
    if not key:
        raise Exception("remote file not ready : %s" % remote_fn)

    # This is the old method which wrote to disk, but won't work with cloud services like Heroku.
    if as_file:
        key.get_contents_to_filename(os.path.join(tmp_dir, fn+'.zip'))
        return zipfile.ZipFile(os.path.join(tmp_dir, fn+'.zip')).open(fn)
    else:
        zipstring = key.get_contents_as_string()
        zipstringio = StringIO(zipstring)
        zf = zipfile.ZipFile(zipstringio).open(fn)
        return zf



from django.db.models import Sum
from .models import BillingRecord
import pprint

def get_cost_for_resource(resource_id):
    cost = BillingRecord.objects.filter(resource_id=resource_id).aggregate(Sum('cost'))['cost__sum']
    return cost

def get_total_cost():
    cost = BillingRecord.objects.all().aggregate(Sum('cost'))['cost__sum']
    return cost

def get_all_resources():
    resources = BillingRecord.objects.values('resource_id').distinct()

    resource_list = []
    for resource in resources:
        resource_list.append(resource.values()[0])

    return resource_list

def get_all_costs_by_resource():
    resources = get_all_resources()
    costs = {}

    for resource in resources:
        costs[resource] = get_cost_for_resource(resource)

    return costs